from django.shortcuts import render, redirect
from django.views import View
from django import http
import re
from .models import User
from django_redis import get_redis_connection
from django.contrib.auth import login, authenticate, logout
from django.contrib.auth.mixins import LoginRequiredMixin


class RegisterView(View):
    # 跳转到注册页面
    def get(self, request):
        return render(request, 'register.html')

    def post(self, request):
        # 接收
        username = request.POST.get('user_name')
        password = request.POST.get('pwd')
        cpwd = request.POST.get('cpwd')
        phone = request.POST.get('phone')
        allow = request.POST.get('allow')
        msg_code = request.POST.get('msg_code')
        pic_code = request.POST.get('pic_code')

        # 验证
        # 1.6个参数是否有空值
        if not all([username, password, cpwd, phone, allow, pic_code]):
            return http.HttpResponseBadRequest('参数不完整')
        # 2.用户名,是否符合正则,是否存在
        if not re.match('^[a-zA-Z0-9_-]{5,20}$', username):
            return http.HttpResponseBadRequest('请输入5-20个字符的用户名')
        if User.objects.filter(username=username).count() > 0:
            return http.HttpResponseBadRequest('用户名已被使用')
        # 3.密码是否符合正则
        if not re.match('[0-9A-Za-z]{8,20}$', password):
            return http.HttpResponseBadRequest('请输入8-20位的密码')
        # 4.两次密码是否一致
        if password != cpwd:
            return http.HttpResponseBadRequest('两次密码输入不一致')
        # 5.手机号是否符合正则,是否存在
        if not re.match('^1[345789]\d{9}', phone):
            return http.HttpResponseBadRequest('手机号不合法')
        if User.objects.filter(mobile=phone).count() > 0:
            return http.HttpResponseBadRequest('手机号已被注册')
        # 7.验证码是否正确
        # 1.连接redis
        redis_cli = get_redis_connection('verify_code')
        # 2.读取redis中的短信验证码字符
        sms_code = redis_cli.get('sms_' + phone)
        # 3.判断是否过期
        if not sms_code:
            return http.HttpResponseBadRequest('验证码已过期')
        # 4.强制立即过期
        redis_cli.delete('sms_' + phone)
        # 5.判断是否正确
        if sms_code.decode() != msg_code:
            return http.HttpResponseBadRequest('短信验证码错误')
        # 处理
        # 创建用户对象,存入数据库
        user = User.objects.create_user(
            username=username, password=password, mobile=phone)
        login(request, user)

        # 响应
        response = redirect('/')
        response.set_cookie('username', user.username, max_age=60 * 60 * 24 * 14)

        return response


class LoginView(View):
    def get(self, request):
        return render(request, 'login.html')

    def post(self, request):
        # 1.接收
        username = request.POST.get('username')
        password = request.POST.get('pwd')

        # 接收重定向参数
        next_url = request.GET.get('next', '/')
        # 2.验证
        # 1.是否有空值
        if not all([username, password]):
            return http.HttpResponseBadRequest('参数不完整')
        # 2.用户名是否符合正则
        if not re.match('^[a-zA-Z0-9_-]{5,20}$', username):
            return http.HttpResponseBadRequest('请输入5-20个字符的用户名')
        # 3.密码是否符合正则
        if not re.match('[0-9A-Za-z]{8,20}$', password):
            return http.HttpResponseBadRequest('请输入8-20位的密码')
        # 4.用户名密码是否正确
        user = authenticate(username=username, password=password)
        if user is None:
            # 用户名或者密码错误
            return render(request, 'login.html', {
                'loginerror': '用户名或者密码错误'
            })
        else:
            # 登录成功,状态保持
            login(request, user)
            # 将用户名保存在cookie中，用于在前端显示登录状态
            response = redirect(next_url)
            response.set_cookie('username', user.username, max_age=60 * 60 * 24 * 14)
            return response

class LogoutView(View):
    def get(self, request):
        # 删除session
        logout(request)
        # 删除cookie
        response = redirect('/')
        response.delete_cookie('username')
        return response

class UserInfoView(LoginRequiredMixin, View):
    def get(self, request):
        return render(request, 'user_center_info.html')
